@@ -1,17 +1,18 @@
 # B::Bytecode.pm - The bytecode compiler (.plc), loaded by ByteLoader
+#
 # Copyright (c) 1994-1999 Malcolm Beattie. All rights reserved.
 # Copyright (c) 2003 Enache Adrian. All rights reserved.
 # Copyright (c) 2008-2011 Reini Urban <rurban@cpan.org>. All rights reserved.
 # This module is free software; you can redistribute and/or modify
 # it under the same terms as Perl itself.
 
-# Reviving 5.6 support here is work in progress:
-#   So far the original is used instead, even if the list of failed tests
-#   is impressive: 3,6,8..10,12,15,16,18,25..28. Pretty broken.
+# Reviving 5.6 support here is work in progress, and not yet enabled.
+# So far the original is used instead, even if the list of failed tests
+# is impressive: 3,6,8..10,12,15,16,18,25..28. Pretty broken.
 
 package B::Bytecode;
 
-our $VERSION = '1.09';
+our $VERSION = '1.11';
 
 #use 5.008;
 use B qw(class main_cv main_root main_start
@@ -48,11 +49,11 @@ use B::Concise;
 
 my $PERL56  = ( $] <  5.008001 );
 my $PERL510 = ( $] >= 5.009005 );
-my $PERL511 = ( $] >= 5.011 );
-my $PERL513 = ( $] >= 5.013002 );
+my $PERL512 = ( $] >= 5.011 );
+#my $PERL514 = ( $] >= 5.013002 );
 my $DEBUGGING = ($Config{ccflags} =~ m/-DDEBUGGING/);
-our ($quiet, %debug);
-my ( $varix, $opix, $savebegins, %walked, %files, @cloop );
+our ($quiet, $includeall, $savebegins, $T_inhinc);
+my ( $varix, $opix, %debug, %walked, %files, @cloop );
 my %strtab  = ( 0, 0 );
 my %svtab   = ( 0, 0 );
 my %optab   = ( 0, 0 );
@@ -220,7 +221,9 @@ sub B::SV::ix {
   defined($ix) ? $ix : do {
     nice '[' . class($sv) . " $tix]";
     B::Assembler::maxsvix($tix) if $debug{A};
-    asm "newsvx", $sv->FLAGS, $debug{Comment} ? sv_flags($sv) : '';
+    my $type = $sv->FLAGS & 0xff; # SVTYPEMASK
+    asm "newsvx", $sv->FLAGS, 
+     $debug{Comment} ? sprintf("type=%d,flags=0x%x,%s", $type, $sv->FLAGS,sv_flags($sv)) : '';
     asm "stsv", $tix if $PERL56;
     $svtab{$$sv} = $varix = $ix = $tix++;
     #nice "\tsvtab ".$$sv." => bsave(".$ix.");
@@ -234,8 +237,10 @@ sub B::GV::ix {
   my $ix = $svtab{$$gv};
   defined($ix) ? $ix : do {
     if ( $debug{G} and !$PERL510 ) {
+      select *STDERR;
       eval "require B::Debug;";
       $gv->B::GV::debug;
+      select *STDOUT;
     }
     if ( ( $PERL510 and $gv->isGV_with_GP )
       or ( !$PERL510 and !$PERL56 and $gv->GP ) )
@@ -413,7 +418,7 @@ sub B::PV::bsave {
 sub B::IV::bsave {
   my ( $sv, $ix ) = @_;
   return $sv->B::RV::bsave($ix)
-    if $PERL511 and $sv->FLAGS & B::SVf_ROK;
+    if $PERL512 and $sv->FLAGS & B::SVf_ROK;
   $sv->B::NULL::bsave($ix);
   if ($PERL56) {
     asm $sv->needs64bits ? "xiv64" : "xiv32", $sv->IVX;
@@ -482,7 +487,7 @@ sub B::PVNV::bsave {
 
 sub B::PVMG::domagic {
   my ( $sv, $ix ) = @_;
-  nice1 '-MAGICAL-'; # XXX TODO no empty line before
+  nice1 '-MAGICAL-'; # no empty line before
   my @mglist = $sv->MAGIC;
   my ( @mgix, @namix );
   for (@mglist) {
@@ -568,19 +573,22 @@ sub B::CV::bsave {
   my $outsideix = $cv->OUTSIDE->ix;
   my $startix   = $cv->START->opwalk;
   my $rootix    = $cv->ROOT->ix;
+  # TODO 5.14 will need CvGV_set to add backref magic
+  my $xsubanyix  = ($cv->CONST and !$PERL56) ? $cv->XSUBANY->ix : 0;
 
   $cv->B::PVMG::bsave($ix);
   asm "xcv_stash",       $stashix;
   asm "xcv_start",       $startix;
   asm "xcv_root",        $rootix;
-  unless ($PERL56) {
-    asm "xcv_xsubany",   $cv->CONST ? $cv->XSUBANY->ix : 0;
-  }
+  asm "xcv_xsubany",     $xsubanyix unless $PERL56;
   asm "xcv_padlist",     $padlistix;
   asm "xcv_outside",     $outsideix;
   asm "xcv_outside_seq", $cv->OUTSIDE_SEQ unless $PERL56;
   asm "xcv_depth",       $cv->DEPTH;
-  asm "xcv_flags",       $cv->CvFLAGS;
+  # add the RC flag if there's no backref magic. eg END (48)
+  my $cvflags = $cv->CvFLAGS;
+  $cvflags |= 0x400 if $] >= 5.013 and !$cv->MAGIC;
+  asm "xcv_flags",       $cvflags;
   asm "xcv_gv",          $gvix;
   asm "xcv_file",        pvix $cv->FILE if $cv->FILE;    # XXX AD
 }
@@ -636,11 +644,12 @@ sub B::AV::bsave {
 sub B::GV::desired {
   my $gv = shift;
   my ( $cv, $form );
-  if ( $debug{G} and !$PERL510 ) {
+  if ( $debug{Gall} and !$PERL510 ) {
+    select *STDERR;	
     eval "require B::Debug;";
     $gv->debug;
+    select *STDOUT;
   }
-  #unless ($] > 5.013005 and $hv->NAME eq 'B')
   $files{ $gv->FILE } && $gv->LINE
     || ${ $cv   = $gv->CV }   && $files{ $cv->FILE }
     || ${ $form = $gv->FORM } && $files{ $form->FILE };
@@ -666,30 +675,13 @@ sub B::HV::bwalk {
       if ($] > 5.013005 and $hv->NAME eq 'B') { # see above. omit B prototypes
 	return;
       }
-      # XXX Not working! Special init for empty (null-string) prototypes
-      # Note: not found constants are &PL_sv_yes, found typically IV
-      if ($PERL510 and 0 and $v->SvTYPE == $SVt_PV and !$v->PVX) {
-        nice "[emptyCONST $tix]";
-        B::Assembler::maxsvix($tix) if $debug{A};
-	asm "newpv", pvstring ($hv->NAME . "::" . $k);
-	# Beware of special gv_fetchpv GV_* flags.
-	# gv_fetchpvx uses only GV_ADD, which fails e.g. with *Fcntl::O_SHLOCK,
-	# if "Your vendor has not defined Fcntl macro O_SHLOCK"
-	asm "gv_fetchpvn_flags", 1 << 7 + $SVt_PV,
-          "f:0x81<<7+t:PV";# GVf_IMPORTED_CV+INTRO
-        $svtab{$$v} = $varix = $tix;
-        asm "sv_flags", $v->FLAGS, ashex($v->FLAGS);
-        $v->bsave( $tix++ );
-        #$tix++;
-      } else {
-        nice "[prototype $tix]";
-        B::Assembler::maxsvix($tix) if $debug{A};
-	asm "gv_fetchpvx", cstring ($hv->NAME . "::" . $k);
-        $svtab{$$v} = $varix = $tix;
-        # we need the sv_flags before, esp. for DEBUGGING asserts
-        asm "sv_flags",  $v->FLAGS, ashex($v->FLAGS);
-        $v->bsave( $tix++ );
-      }
+      nice "[prototype $tix]";
+      B::Assembler::maxsvix($tix) if $debug{A};
+      asm "gv_fetchpvx", cstring ($hv->NAME . "::" . $k);
+      $svtab{$$v} = $varix = $tix;
+      # we need the sv_flags before, esp. for DEBUGGING asserts
+      asm "sv_flags",  $v->FLAGS, ashex($v->FLAGS);
+      $v->bsave( $tix++ );
     }
   }
 }
@@ -936,7 +928,7 @@ sub B::PADOP::bsave {
   $op->B::OP::bsave($ix);
 
   # XXX crashed in 5.11 (where, why?)
-  #if ($PERL511) {
+  #if ($PERL512) {
   asm "op_padix", $op->padix;
   #}
 }
@@ -1010,9 +1002,19 @@ sub B::OP::opwalk {
   }
 }
 
-sub save_cq {
+# Do run-time requires with -s savebegin and without -i includeall.
+# Otherwise all side-effects of BEGIN blocks are already in the current
+# compiled code.
+# -s or !-i will have smaller code, but run-time access of dependent modules
+# such as with python, where all modules are byte-compiled.
+# With -i the behaviour is similar to the C or CC compiler, where everything
+# is packed into one file.
+# Redo only certain ops, such as push @INC ""; unshift @INC "" (TODO *INC)
+# use/require defs and boot sections are already included.
+sub save_begin {
   my $av;
-  if ( ( $av = begin_av )->isa("B::AV") ) {
+  if ( ( $av = begin_av )->isa("B::AV") and $av->ARRAY) {
+    nice '<push_begin>';
     if ($savebegins) {
       for ( $av->ARRAY ) {
         next unless $_->FILE eq $0;
@@ -1025,36 +1027,45 @@ sub save_cq {
 
         # XXX BEGIN { goto A while 1; A: }
         for ( my $op = $_->START ; $$op ; $op = $op->next ) {
-	  # special cases for:
 	  # 1. push|unshift @INC, "libpath"
-	  if ($op->name =~ /^(unshift|push)$/) {
-	    asm "push_begin", $_->ix;
-	    last;
+	  if ($op->name eq 'gv') {
+            my $gv = class($op) eq 'SVOP'
+                  ? $op->gv
+                  : ( ( $_->PADLIST->ARRAY )[1]->ARRAY )[ $op->padix ];
+	    nice1 '<gv '.$gv->NAME.'>' if $$gv;
+            asm "incav", inc_gv->AV->ix if $$gv and $gv->NAME eq 'INC'; 
 	  }
-	  # 2. use|require ... unless in tests
-          next unless $op->name eq 'require' ||
-
+	  # 2. use|require
+	  if (!$includeall) {
+	    next unless $op->name eq 'require' ||
               # this kludge needed for tests
               $op->name eq 'gv' && do {
-                my $gv =
-                  class($op) eq 'SVOP'
+                my $gv = class($op) eq 'SVOP'
                   ? $op->gv
                   : ( ( $_->PADLIST->ARRAY )[1]->ARRAY )[ $op->padix ];
                 $$gv && $gv->NAME =~ /use_ok|plan/;
               };
-          asm "push_begin", $_->ix;
-          last;
+              nice1 '<require in BEGIN>';
+              asm "push_begin", $_->ix if $_;
+              last;
+	   }
         }
       }
     }
   }
-  if ( ( $av = init_av )->isa("B::AV") ) {
+}
+
+sub save_init_end {
+  my $av;
+  if ( ( $av = init_av )->isa("B::AV") and $av->ARRAY ) {
+    nice '<push_init>';
     for ( $av->ARRAY ) {
       next unless $_->FILE eq $0;
       asm "push_init", $_->ix;
     }
   }
-  if ( ( $av = end_av )->isa("B::AV") ) {
+  if ( ( $av = end_av )->isa("B::AV") and $av->ARRAY ) {
+    nice '<push_end>';
     for ( $av->ARRAY ) {
       next unless $_->FILE eq $0;
       asm "push_end", $_->ix;
@@ -1091,9 +1102,14 @@ sub symwalk {
 ################### end perl 5.6 backport ###################################
 
 sub compile {
-  my ( $head, $scan, $T_inhinc, $keep_syn, $module );
+  my ( $head, $scan, $keep_syn, $module );
   my $cwd = '';
   $files{$0} = 1;
+  # includeall mode (without require):
+  if ($includeall) {
+    # add imported symbols => values %INC
+    $files{$_} = 1 for values %INC;
+  }
 
   sub keep_syn {
     $keep_syn         = 1;
@@ -1147,6 +1163,9 @@ use ByteLoader '$ByteLoader::VERSION';
     elsif (/^-f(.*)$/) {
       $files{$1} = 1;
     }
+    elsif (/^-i/) {
+      $includeall = 1;
+    }
     elsif (/^-D(.*)$/) {
       $debug{$1}++;
     }
@@ -1201,6 +1220,11 @@ use ByteLoader '$ByteLoader::VERSION';
     print $head if $head;
     newasm sub { print @_ };
 
+    nice '<incav>' if $T_inhinc;
+    asm "incav", inc_gv->AV->ix if $T_inhinc;
+    save_begin;
+    #asm "incav", inc_gv->AV->ix if $T_inhinc;
+    nice '<end_begin>';
     if (!$PERL56) {
       defstash->bwalk;
     } else {
@@ -1229,10 +1253,7 @@ use ByteLoader '$ByteLoader::VERSION';
 
     asm "signal", cstring "__WARN__"    # XXX
       if !$PERL56 and warnhook->ix;
-    nice '<incav>';
-    asm "incav", inc_gv->AV->ix if $T_inhinc;
-    save_cq;
-    asm "incav", inc_gv->AV->ix if $T_inhinc;
+    save_init_end;
     asm "dowarn", dowarn unless $PERL56;
 
     {
@@ -1283,8 +1304,17 @@ the sourcecode in memory.
 =item B<-H>
 
 Prepend a C<use ByteLoader VERSION;> line to the produced bytecode.
+This way you will not need to add C<-MByteLoader> to your perl command-line.
 
-=item B<-b>
+=item B<-i> includeall
+
+Include all used packages and its symbols. Does no run-time require from
+BEGIN blocks (C<use> package).
+
+This creates bigger and more independent code, but is more error prone and
+does not support pre-compiled C<.pmc> modules.
+
+=item B<-b> savebegin
 
 Save all the BEGIN blocks.
 
@@ -1322,6 +1352,11 @@ C<main_root>, C<main_cv> and C<curpad> are omitted.
 
 "use package." Might be needed of the package is not automatically detected.
 
+=item B<-f>I<file>
+
+Include file. If not C<-i> define all symbols in the given included
+source file. C<-i> would all included files, C<-f> only a certain file - full path needed.
+
 =item B<-q>
 
 Be quiet.
@@ -1342,17 +1377,17 @@ Set the COP file - for running within the CORE testsuite.
 
 OPs, prints each OP as it's processed
 
-=item B<-D>I<M>
+=item B<-DM>
 
 Debugging flag for more verbose STDERR output.
 
 B<M> for Magic and Matches.
 
-=item B<-D>I<G>
+=item B<-DG>
 
 Debug GV's
 
-=item B<-D>I<A>
+=item B<-DA>
 
 Set developer B<A>ssertions, to help find possible obj-indices out of range.
 
@@ -1393,8 +1428,6 @@ Special GV's fail.
 
 There are also undocumented bugs and options.
 
-THIS CODE IS HIGHLY EXPERIMENTAL. USE AT YOUR OWN RISK.
-
 =head1 AUTHORS
 
 Originally written by Malcolm Beattie 1996 and
